# -*- coding: utf-8 -*-


from PyQt4 import QtCore, QtGui


def isExpandable(modelIndex):
    return modelIndex.model().rowCount(modelIndex)>0


def isSelectable(modelIndex):
    flags = modelIndex.flags()
    return flags & QtCore.Qt.ItemIsEnabled and flags & QtCore.Qt.ItemIsSelectable


class CTreeComboBoxView(QtGui.QTreeView):
    def __init__(self, parent=None):
        QtGui.QTreeView.__init__(self, parent)
        self.setUniformRowHeights(True)
        self.setRootIsDecorated(True)
        self.setItemsExpandable(True)
        self.setSelectionBehavior(QtGui.QTreeView.SelectRows)
        self.header().setVisible(False)
        self.setMinimumHeight(150)


    def expandAllFromIndex(self, index):
        blockSignals = self.blockSignals(True)
        try:
            rowCount = self.model().rowCount
            toExpand = [ index ]
            while toExpand:
                index = toExpand.pop()
                if index.isValid():
                    self.expand(index)
                    for i in xrange(rowCount(index)):
                        toExpand.append( index.child(i, 0) )
        finally:
            self.blockSignals(blockSignals)


    def expandUp(self, index):
        blockSignals = self.blockSignals(True)
        try:
            self.expandToDepth(0)
            if index and index.isValid():
                index = index.parent()
                while index and index.isValid():
                    self.expand(index)
                    index = index.parent()
        finally:
            self.blockSignals(blockSignals)


    def conditionalShiftUp(self, event, index):
        if event.modifiers() & QtCore.Qt.ShiftModifier:
            self.scrollTo(index, QtGui.QAbstractItemView.PositionAtTop)



class CTreeComboBoxBase(QtGui.QComboBox):
    # предполагается, что в наследниках этого класса
    # достаточно написать свою модель + сделать self.setModel
    def __init__(self, parent=None):
        QtGui.QComboBox.__init__(self, parent)
        self._view = CTreeComboBoxView()
        self._currentModelIndex = None
        self._selectedModelIndex = None
        self._expandAll = False
        self.setView(self._view)
        self._view.installEventFilter(self)
        self._view.viewport().installEventFilter(self)


    def setExpandAll(self, value):
        self._expandAll = value


    def eventFilter(self, watched, event):
        view = self._view

        if event.type() == QtCore.QEvent.MouseButtonPress and watched == view.viewport():
            pos = event.pos()
            index = view.indexAt(pos)
            if index.isValid() and not view.visualRect(index).contains(pos):
                if isExpandable(index):
                    view.setExpanded(index, not view.isExpanded(index))
                    if view.isExpanded(index):
                        view.conditionalShiftUp(event, index)
            return True # filter out
        elif event.type() == QtCore.QEvent.MouseButtonRelease and watched == view.viewport():
            pos = event.pos()
            index = view.indexAt(pos)
            if view.visualRect(index).contains(pos) and isSelectable(index):
                self._selectedModelIndex = index
                self.hidePopup()
            return True
        if event.type() == QtCore.QEvent.KeyPress and watched == view:
            key = event.key()
            index = view.currentIndex()
            if key == QtCore.Qt.Key_Minus:
                view.collapse(index)
                return True
            elif key == QtCore.Qt.Key_Plus:
                if isExpandable(index):
                    view.expand(index)
                    view.conditionalShiftUp(event, index)
                return True
            elif key == QtCore.Qt.Key_Asterisk:
                if isExpandable(index):
                    view.expandAllFromIndex(index)
                    view.conditionalShiftUp(event, index)
                return True
            elif key in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return, QtCore.Qt.Key_Select):
                if isSelectable(index):
                    self._selectedModelIndex = view.currentIndex()
                    self.hidePopup()
                return True
            elif key in (QtCore.Qt.Key_Escape, QtCore.Qt.Key_F4):
                self.hidePopup()
                return True
        return False


    def showPopup(self):
        self.setRootModelIndex(QtCore.QModelIndex())
        if self._expandAll:
            self._view.expandAll()
        else:
            self._view.expandUp(self.currentModelIndex())
        self._selectedModelIndex = None
        QtGui.QComboBox.showPopup(self)
        scrollBar = self._view.horizontalScrollBar()
        scrollBar.setValue(0)


    def hidePopup(self):
        if self._view.isVisible():
            QtGui.QComboBox.hidePopup(self)
            if self._selectedModelIndex:
                self.setCurrentModelIndex(self._selectedModelIndex)
                self._selectedModelIndex = None


    def currentModelIndex(self):
        return QtCore.QModelIndex(self._currentModelIndex) if self._currentModelIndex else QtCore.QModelIndex()


    def setCurrentModelIndex(self, modelIndex):
        if modelIndex and modelIndex.isValid():
            if modelIndex != self._currentModelIndex:
                self._currentModelIndex = QtCore.QPersistentModelIndex(modelIndex)
                self.setRootModelIndex(modelIndex.parent())
                self.setCurrentIndex(modelIndex.row())
        else:
            self._currentModelIndex = None
            self.setRootModelIndex(QtCore.QModelIndex())
            self.setCurrentIndex(0)
