# -*- coding: utf-8 -*-

from PyQt4 import QtCore, QtGui

class CTrivialTreeModel(QtCore.QAbstractItemModel):
    # we store in QModelIndex.internalId 
    # zero-based index in N-ary heap of items

    maxDepth = 4
    childCount = 4
    maxKey = (childCount**maxDepth-1)/(childCount-1)-1


    def __init__(self, parent, rootItem=None):
        QtCore.QAbstractItemModel.__init__(self, parent)
        self.loop = self # actual model has an indirect cycle


    def _parentKey(self, key):
        if key>0:
            return (key-1) // self.childCount
        else:
            return 0


    def _childKey(self, key, row):
        if key>=0:
            return (key * self.childCount) + row + 1
        else:
            return 0


    def _rowFromKey(self, key):
        if key>0:
            return (key-1) % self.childCount
        else:
            return 0


    def columnCount(self, parent=None):
        return 1


    def rowCount(self, parent):
        if parent and parent.isValid():
            key = parent.internalId()
            if key > self.maxKey:
                return 0
            else:
                return self.childCount
        else:
            return 1


    def parent(self, index):
        key = index.internalId()
        if key>0:
            parentKey = self._parentKey(key)
            return self.createIndex( self._rowFromKey(parentKey), 0, parentKey)
        return QtCore.QModelIndex()


    def index(self, row, column, parent = QtCore.QModelIndex()):
        if parent and parent.isValid():
            parentKey = parent.internalId()
            key = self._childKey(parentKey, row)
            return self.createIndex(row, column, key)
        else:
            return self.createIndex(0, 0, 0)


    def data(self, index, role):
        if index.isValid() and role == QtCore.Qt.DisplayRole:
            key = index.internalId()
            return QtCore.QVariant(key)
        return QtCore.QVariant()


    def flags(self, index):
        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
